@@ -4,6 +4,7 @@ Snap7 client used for connection to a siemens7 server.
 import re
 from ctypes import c_int, c_char_p, byref, sizeof, c_uint16, c_int32, c_byte
 from ctypes import c_void_p
+from datetime import datetime
 
 import logging
 
@@ -39,7 +40,7 @@ class Client(object):
 
     def __del__(self):
         self.destroy()
-        
+
     def create(self):
         """
         create a SNAP7 client.
@@ -83,15 +84,15 @@ class Client(object):
         """
         state = c_int(0)
         self.library.Cli_GetPlcStatus(self.pointer,byref(state))
-        
+
         try:
             status_string = cpu_statuses[state.value]
         except KeyError:
             status_string = None
-        
+
         if not status_string:
             raise Snap7Exception("The cpu state (%s) is invalid" % state.value)
-        
+
         logger.debug("CPU state is %s" % status_string)
         return status_string
 
@@ -164,7 +165,7 @@ class Client(object):
     def delete(self, block_type, block_num):
         """
         Deletes a block
-        
+
         :param block_type: Type of block
         :param block_num: Bloc number
         """
@@ -172,7 +173,7 @@ class Client(object):
         blocktype = snap7.snap7types.block_types[block_type]
         result = self.library.Cli_Delete(self.pointer, blocktype, block_num)
         return result
-      
+
     def full_upload(self, _type, block_num):
         """
         Uploads a full block body from AG.
@@ -246,7 +247,12 @@ class Client(object):
         :param size: number of units to read
         """
         assert area in snap7.snap7types.areas.values()
-        wordlen = snap7.snap7types.S7WLByte
+        if area == snap7.snap7types.S7AreaTM:
+            wordlen = snap7.snap7types.S7WLTimer
+        elif area == snap7.snap7types.S7AreaCT:
+            wordlen = snap7.snap7types.S7WLCounter
+        else:
+            wordlen = snap7.snap7types.S7WLByte
         type_ = snap7.snap7types.wordlen_to_ctypes[wordlen]
         logger.debug("reading area: %s dbnumber: %s start: %s: amount %s: "
                       "wordlen: %s" % (area, dbnumber, start, size, wordlen))
@@ -267,11 +273,16 @@ class Client(object):
         :param start: offset to start writing
         :param data: a bytearray containing the payload
         """
-        wordlen = snap7.snap7types.S7WLByte
-        type_ = snap7.snap7types.wordlen_to_ctypes[wordlen]
+        if area == snap7.snap7types.S7AreaTM:
+            wordlen = snap7.snap7types.S7WLTimer
+        elif area == snap7.snap7types.S7AreaCT:
+            wordlen = snap7.snap7types.S7WLCounter
+        else:
+            wordlen = snap7.snap7types.S7WLByte
+        type_ = snap7.snap7types.wordlen_to_ctypes[snap7.snap7types.S7WLByte]
         size = len(data)
         logger.debug("writing area: %s dbnumber: %s start: %s: size %s: "
-                      "type: %s" % (area, dbnumber, start, size, type_))
+                     "wordlen %s type: %s" % (area, dbnumber, start, size, wordlen, type_))
         cdata = (type_ * len(data)).from_buffer_copy(data)
         return self.library.Cli_WriteArea(self.pointer, area, dbnumber, start,
                                           size, wordlen, byref(cdata))
@@ -312,7 +323,7 @@ class Client(object):
 
         if (size == 0):
             return 0
-		
+
         data = (c_uint16 * size)()
         count = c_int(size)
         result = self.library.Cli_ListBlocksOfType(
@@ -575,3 +586,41 @@ class Client(object):
         check_error(code)
 
         return negotiated_.value
+
+    def get_plc_datetime(self):
+        """
+        Get date and time from PLC.
+
+        :return: date and time as datetime
+        """
+        type_ = c_int32
+        buffer = (type_ * 9)()
+        result = self.library.Cli_GetPlcDateTime(self.pointer, byref(buffer))
+        check_error(result, context="client")
+
+        return datetime(
+            year = buffer[5] + 1900,
+            month = buffer[4] + 1,
+            day = buffer[3],
+            hour = buffer[2],
+            minute = buffer[1],
+            second = buffer[0]
+        )
+
+    @error_wrap
+    def set_plc_datetime(self, dt):
+        """
+        Set date and time in PLC
+
+        :param dt: date and time as datetime
+        """
+        type_ = c_int32
+        buffer = (type_ * 9)()
+        buffer[0] = dt.second
+        buffer[1] = dt.minute
+        buffer[2] = dt.hour
+        buffer[3] = dt.day
+        buffer[4] = dt.month - 1
+        buffer[5] = dt.year - 1900
+
+        return self.library.Cli_SetPlcDateTime(self.pointer, byref(buffer))
