@@ -212,7 +212,7 @@ def get_string(_bytearray, byte_index, max_size):
     size = _bytearray[byte_index + 1]
 
     if max_size < size:
-        logger.error("the string is to big for the size encountered in specification")
+        logger.error("the string is too big for the size encountered in specification")
         logger.error("WRONG SIZED STRING ENCOUNTERED")
         size = max_size
 
