@@ -5,11 +5,12 @@ import logging
 import time
 import mock
 
+from datetime import datetime
 from subprocess import Popen
 from os import path, kill
 import snap7
 from snap7.snap7exceptions import Snap7Exception
-from snap7.snap7types import S7AreaDB, S7WLByte, S7DataItem
+from snap7.snap7types import S7AreaDB, S7WLByte, S7DataItem, S7AreaTM, S7AreaCT
 from snap7 import util
 
 
@@ -143,19 +144,41 @@ class TestClient(unittest.TestCase):
         self.client.download(block_num=db_number, data=data)
 
     def test_read_area(self):
-        area = snap7.snap7types.areas.DB
-        dbnumber = 1
         amount = 1
         start = 1
+        # Test read_area with a DB
+        area = snap7.snap7types.areas.DB
+        dbnumber = 1
+        self.client.read_area(area, dbnumber, start, amount)
+        # Test read_area with a TM
+        area = snap7.snap7types.areas.TM
+        dbnumber = 0
+        self.client.read_area(area, dbnumber, start, amount)
+        # Test read_area with a CT
+        area = snap7.snap7types.areas.CT
+        dbnumber = 0
         self.client.read_area(area, dbnumber, start, amount)
 
     def test_write_area(self):
+        # Test write area with a DB
         area = snap7.snap7types.areas.DB
         dbnumber = 1
         size = 1
         start = 1
         data = bytearray(size)
         self.client.write_area(area, dbnumber, start, data)
+        # Test write area with a TM
+        area = snap7.snap7types.areas.TM
+        dbnumber = 0
+        size = 2
+        timer = bytearray(size)
+        self.client.write_area(area, dbnumber, start, timer)
+        # Test write area with a CT
+        area = snap7.snap7types.areas.CT
+        dbnumber = 0
+        size = 2
+        timer = bytearray(size)
+        self.client.write_area(area, dbnumber, start, timer)
 
     def test_list_blocks(self):
         blockList = self.client.list_blocks()
@@ -444,6 +467,14 @@ class TestClient(unittest.TestCase):
         finally:
             self.client.library.Cli_AsDownload = original
 
+    def test_get_plc_time(self):
+        self.assertEqual(datetime.now().replace(microsecond=0), self.client.get_plc_datetime())
+
+    def test_set_plc_datetime(self):
+        new_dt = datetime(2011,1,1,1,1,1,0)
+        self.client.set_plc_datetime(new_dt)
+        # Can't actual set datetime in emulated PLC, get_plc_datetime always returns system time.
+        #self.assertEqual(new_dt, self.client.get_plc_datetime())
 
 
 class TestClientBeforeConnect(unittest.TestCase):
@@ -531,7 +562,6 @@ Cli_GetOrderCode
 Cli_GetParam
 Cli_GetPduLength
 Cli_GetPgBlockInfo
-Cli_GetPlcDateTime
 Cli_GetPlcStatus
 Cli_GetProtection
 Cli_IsoExchangeBuffer
@@ -543,7 +573,6 @@ Cli_ReadSZL
 Cli_ReadSZLList
 Cli_SetAsCallback
 Cli_SetParam
-Cli_SetPlcDateTime
 Cli_SetPlcSystemDateTime
 Cli_SetSessionPassword
 Cli_TMRead
